"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cloneSchedule = exports.printMoney = exports.getBillingsPerDuration = exports.getMiddleOfDay = exports.getEndDate = exports.getNextCyclesStartDate = exports.getNextStartDate = exports.getDaysUntilNextStartDate = exports.getDayRate = exports.applySubscriptionPromotions = exports.calculateSubscriptionPricing = void 0;
const core_1 = require("@vendure/core");
const config_helpers_1 = require("@vendure/core/dist/config/config-helpers");
const date_fns_1 = require("date-fns");
const constants_1 = require("../constants");
const graphql_1 = require("../ui/generated/graphql");
const subscription_promotion_1 = require("./subscription.promotion");
/**
 * Calculate subscription pricing based on variants, schedules and optional input
 * Doesn't apply discounts yet!
 *
 * @param ctx
 * @param rawSubscriptionPrice
 * @param schedule The schedule that should be used for the Subscription
 * @param input
 * @returns
 */
function calculateSubscriptionPricing(ctx, rawSubscriptionPrice, schedule, input) {
    let downpayment = schedule.downpayment;
    if (input?.downpayment || input?.downpayment === 0) {
        downpayment = input.downpayment;
    }
    if (schedule.paidUpFront && schedule.downpayment) {
        // Paid-up-front subscriptions cant have downpayments
        throw new core_1.UserInputError(`Paid-up-front subscriptions can not have downpayments!`);
    }
    if (schedule.paidUpFront && downpayment) {
        throw new core_1.UserInputError(`You can not use downpayments with Paid-up-front subscriptions`);
    }
    const billingsPerDuration = getBillingsPerDuration(schedule);
    const totalSubscriptionPrice = rawSubscriptionPrice * billingsPerDuration + schedule.downpayment;
    if (downpayment > totalSubscriptionPrice) {
        throw new core_1.UserInputError(`Downpayment cannot be higher than the total subscription value, which is (${printMoney(totalSubscriptionPrice)})`);
    }
    if (downpayment < schedule.downpayment) {
        throw new core_1.UserInputError(`Downpayment cannot be lower than schedules default downpayment, which is (${printMoney(schedule.downpayment)})`);
    }
    if (schedule.startMoment === graphql_1.SubscriptionStartMoment.FixedStartdate &&
        input?.startDate) {
        throw new core_1.UserInputError('You cannot choose a custom startDate for schedules with a fixed start date');
    }
    const dayRate = getDayRate(totalSubscriptionPrice, schedule.durationInterval, schedule.durationCount);
    const now = new Date();
    let subscriptionStartDate = getNextStartDate(now, schedule.billingInterval, schedule.startMoment, schedule.fixedStartDate);
    let subscriptionEndDate = undefined;
    if (!schedule.autoRenew) {
        // Without autorenewal we need an endDate
        subscriptionEndDate = getEndDate(subscriptionStartDate, schedule.startMoment, schedule.durationInterval, schedule.durationCount);
    }
    let proratedDays = 0;
    let totalProratedAmount = 0;
    if (schedule.useProration) {
        proratedDays = getDaysUntilNextStartDate(input?.startDate || now, subscriptionStartDate);
        totalProratedAmount = proratedDays * dayRate;
    }
    let amountDueNow = downpayment + totalProratedAmount;
    let recurringPrice = Math.floor((totalSubscriptionPrice - downpayment) / billingsPerDuration);
    if (schedule.paidUpFront) {
        // User pays for the full membership now
        amountDueNow = rawSubscriptionPrice + totalProratedAmount;
        recurringPrice = rawSubscriptionPrice;
        // If paid up front, move the startDate to next cycle, because the first cycle has been paid up front. This needs to happen AFTER proration calculation
        subscriptionStartDate = getNextCyclesStartDate(new Date(), schedule.startMoment, schedule.durationInterval, schedule.durationCount);
    }
    return {
        downpayment: downpayment,
        totalProratedAmount: totalProratedAmount,
        proratedDays: proratedDays,
        dayRate,
        pricesIncludeTax: ctx.channel.pricesIncludeTax,
        recurringPrice: recurringPrice,
        interval: schedule.billingInterval,
        intervalCount: schedule.billingCount,
        amountDueNow,
        subscriptionStartDate,
        subscriptionEndDate,
        schedule: cloneSchedule(ctx, schedule),
    };
}
exports.calculateSubscriptionPricing = calculateSubscriptionPricing;
/**
 * Calculate the discounted recurring price based on given promotions
 */
async function applySubscriptionPromotions(ctx, recurringPrice, orderLine, promotions) {
    let discountedRecurringPrice = recurringPrice;
    const allActions = (0, config_helpers_1.getConfig)().promotionOptions.promotionActions || [];
    for (const promotion of promotions) {
        for (const action of promotion.actions) {
            const promotionAction = allActions.find((a) => a.code === action.code);
            if (promotionAction instanceof subscription_promotion_1.SubscriptionPromotionAction) {
                const discount = await promotionAction.executeOnSubscription(ctx, discountedRecurringPrice, orderLine, action.args);
                const newDiscountedPrice = discountedRecurringPrice + discount;
                core_1.Logger.info(`Discounted recurring price from ${discountedRecurringPrice} to ${newDiscountedPrice} for promotion ${promotion.name}`, constants_1.loggerCtx);
                discountedRecurringPrice = newDiscountedPrice;
            }
        }
    }
    return discountedRecurringPrice;
}
exports.applySubscriptionPromotions = applySubscriptionPromotions;
/**
 * Calculate day rate based on the total price and duration of the subscription
 * Example: $200 per 6 months
 *          = $400 per 12 months
 *          $400 / 365 = $1,10 per day
 */
function getDayRate(totalPrice, durationInterval, durationCount) {
    let intervalsPerYear = 12; // Default is 1 month
    if (durationInterval === graphql_1.SubscriptionInterval.Week) {
        intervalsPerYear = 52;
    }
    const pricePerYear = (intervalsPerYear / durationCount) * totalPrice;
    return Math.round(pricePerYear / 365);
}
exports.getDayRate = getDayRate;
function getDaysUntilNextStartDate(now, nextStartDate) {
    const startOfToday = (0, date_fns_1.startOfDay)(now);
    return (0, date_fns_1.differenceInDays)(nextStartDate, startOfToday);
}
exports.getDaysUntilNextStartDate = getDaysUntilNextStartDate;
/**
 * Get the next startDate for a given start moment (first or last of the Interval). Always returns the middle of the day for billing
 */
function getNextStartDate(now, interval, startMoment, fixedStartDate) {
    if (startMoment === graphql_1.SubscriptionStartMoment.TimeOfPurchase) {
        return now;
    }
    else if (startMoment === graphql_1.SubscriptionStartMoment.FixedStartdate) {
        if (!fixedStartDate) {
            throw Error(`With a 'Fixed start date' startMoment, the 'fixedStartDate' argument is mandatory`);
        }
        return fixedStartDate;
    }
    let nextStartDate = new Date();
    if (interval === graphql_1.SubscriptionInterval.Month) {
        if (startMoment === graphql_1.SubscriptionStartMoment.StartOfBillingInterval) {
            const nextMonth = (0, date_fns_1.addMonths)(now, 1);
            nextStartDate = (0, date_fns_1.startOfMonth)(nextMonth);
        }
        else if (startMoment === graphql_1.SubscriptionStartMoment.EndOfBillingInterval) {
            nextStartDate = (0, date_fns_1.endOfMonth)(now);
        }
        else {
            throw Error(`Unhandled combination of startMoment=${startMoment} and interval=${interval}`);
        }
    }
    else if (interval === graphql_1.SubscriptionInterval.Week) {
        if (startMoment === graphql_1.SubscriptionStartMoment.StartOfBillingInterval) {
            const nextWeek = (0, date_fns_1.addWeeks)(now, 1);
            nextStartDate = (0, date_fns_1.startOfWeek)(nextWeek);
        }
        else if (startMoment === graphql_1.SubscriptionStartMoment.EndOfBillingInterval) {
            nextStartDate = (0, date_fns_1.endOfWeek)(now);
        }
        else {
            throw Error(`Unhandled combination of startMoment=${startMoment} and interval=${interval}`);
        }
    }
    return getMiddleOfDay(nextStartDate);
}
exports.getNextStartDate = getNextStartDate;
/**
 * Get the next cycles startDate. Used for paid-up-front subscriptions, where a user already paid for the first cycle
 * and we need the next cycles start date
 */
function getNextCyclesStartDate(now, startMoment, interval, intervalCount, fixedStartDate) {
    const endDate = getEndDate(now, startMoment, interval, intervalCount);
    return getNextStartDate(endDate, interval, startMoment, fixedStartDate);
}
exports.getNextCyclesStartDate = getNextCyclesStartDate;
/**
 * Get the endDate of a subscription based on the duration
 */
function getEndDate(startDate, startMoment, interval, intervalCount) {
    let endDate = new Date(startDate);
    if (interval === graphql_1.SubscriptionInterval.Month) {
        endDate = (0, date_fns_1.addMonths)(endDate, intervalCount);
    }
    else {
        // Week
        endDate = (0, date_fns_1.addWeeks)(endDate, intervalCount);
    }
    return endDate;
}
exports.getEndDate = getEndDate;
/**
 * Return the middle of the day (13:00) for dates, because that makes more sense for billing
 */
function getMiddleOfDay(date) {
    const start = new Date(date);
    start.setHours(13, 0, 0, 0);
    return start;
}
exports.getMiddleOfDay = getMiddleOfDay;
/**
 * Get the number of billings per full duration of the schedule
 */
function getBillingsPerDuration(schedule) {
    if (schedule.durationInterval === graphql_1.SubscriptionInterval.Week &&
        schedule.billingInterval === graphql_1.SubscriptionInterval.Month) {
        throw new core_1.UserInputError(`Billing interval must be greater or equal to duration interval. E.g. billing cannot occur monthly for a schedule with a duration of 3 weeks.`);
    }
    if (schedule.billingInterval === schedule.durationInterval) {
        return schedule.durationCount / schedule.billingCount;
    }
    if (schedule.billingInterval === graphql_1.SubscriptionInterval.Week &&
        schedule.durationInterval === graphql_1.SubscriptionInterval.Month) {
        return (4 / schedule.billingCount) * schedule.durationCount;
    }
    throw Error(`Can not calculate billingsPerDurations for billingInterval ${schedule.billingInterval} and durationInterval ${schedule.durationInterval}`);
}
exports.getBillingsPerDuration = getBillingsPerDuration;
/**
 * Yes, it's real, this helper function prints money for you!
 */
function printMoney(amount) {
    return (amount / 100).toFixed(2);
}
exports.printMoney = printMoney;
function cloneSchedule(ctx, schedule) {
    return Object.assign(schedule, {
        pricesIncludeTax: ctx.channel.pricesIncludeTax,
    });
}
exports.cloneSchedule = cloneSchedule;
