"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.subscriptionPromotions = exports.selectedProductsByFixedAmount = exports.selectedProductsByPercentage = exports.withFacetsByFixedAmount = exports.withFacetsByPercentage = exports.allByPercentage = exports.SubscriptionPromotionAction = void 0;
const core_1 = require("@vendure/core");
/**
 * A custom Promotion Action that can discount specific subscription items in an order.
 */
class SubscriptionPromotionAction extends core_1.PromotionOrderAction {
    constructor(config) {
        super({
            ...config,
            execute: () => 0, // No discounts on actual order prices
        });
        this.executeOnSubscriptionFn = config.executeOnSubscription;
    }
    executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        return this.executeOnSubscriptionFn(ctx, currentSubscriptionPrice, orderLine, this.argsArrayToHash(args));
    }
}
exports.SubscriptionPromotionAction = SubscriptionPromotionAction;
exports.allByPercentage = new SubscriptionPromotionAction({
    code: 'discount_all_subscription_payments_by_percentage',
    description: [
        {
            languageCode: core_1.LanguageCode.en,
            value: 'Discount all subscription payments by { discount } %',
        },
    ],
    args: {
        discount: {
            type: 'float',
            ui: {
                component: 'number-form-input',
                suffix: '%',
            },
        },
    },
    async executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        const discount = currentSubscriptionPrice * (args.discount / 100);
        return -discount;
    },
});
let facetValueChecker;
exports.withFacetsByPercentage = new SubscriptionPromotionAction({
    code: 'discount_subscription_payments_with_facets_by_percentage',
    description: [
        {
            languageCode: core_1.LanguageCode.en,
            value: 'Discount subscription payments with facets by { discount } %',
        },
    ],
    args: {
        discount: {
            type: 'float',
            ui: {
                component: 'number-form-input',
                suffix: '%',
            },
        },
        facets: {
            type: 'ID',
            list: true,
            ui: { component: 'facet-value-form-input' },
        },
    },
    init(injector) {
        facetValueChecker = new core_1.FacetValueChecker(injector.get(core_1.TransactionalConnection));
    },
    async executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        if (await facetValueChecker.hasFacetValues(orderLine, args.facets, ctx)) {
            const discount = currentSubscriptionPrice * (args.discount / 100);
            return -discount;
        }
        return 0;
    },
});
exports.withFacetsByFixedAmount = new SubscriptionPromotionAction({
    code: 'discount_subscription_payments_with_facets_by_fixed_amount',
    description: [
        {
            languageCode: core_1.LanguageCode.en,
            value: 'Discount subscription payments with facets by fixed amount',
        },
    ],
    args: {
        amount: {
            type: 'int',
            ui: {
                component: 'currency-form-input',
            },
        },
        facets: {
            type: 'ID',
            list: true,
            ui: { component: 'facet-value-form-input' },
        },
    },
    init(injector) {
        facetValueChecker = new core_1.FacetValueChecker(injector.get(core_1.TransactionalConnection));
    },
    async executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        if (await facetValueChecker.hasFacetValues(orderLine, args.facets, ctx)) {
            const discount = -Math.min(args.amount, currentSubscriptionPrice); // make sure we don't discount more than the current price
            return discount;
        }
        return 0;
    },
});
function lineContainsIds(ids, line) {
    return !!ids.find((id) => id == line.productVariant.id);
}
exports.selectedProductsByPercentage = new SubscriptionPromotionAction({
    code: 'discount_subscription_payments_for_selected_products_by_percentage',
    description: [
        {
            languageCode: core_1.LanguageCode.en,
            value: 'Discount subscription payments for selected products by percentage',
        },
    ],
    args: {
        discount: {
            type: 'float',
            ui: {
                component: 'number-form-input',
                suffix: '%',
            },
        },
        productVariantIds: {
            type: 'ID',
            list: true,
            ui: { component: 'product-selector-form-input' },
            label: [{ languageCode: core_1.LanguageCode.en, value: 'Product variants' }],
        },
    },
    init(injector) {
        facetValueChecker = new core_1.FacetValueChecker(injector.get(core_1.TransactionalConnection));
    },
    async executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        if (lineContainsIds(args.productVariantIds, orderLine)) {
            const discount = currentSubscriptionPrice * (args.discount / 100);
            return -discount;
        }
        return 0;
    },
});
exports.selectedProductsByFixedAmount = new SubscriptionPromotionAction({
    code: 'discount_subscription_payments_for_selected_products_by_fixed_amount',
    description: [
        {
            languageCode: core_1.LanguageCode.en,
            value: 'Discount subscription payments for selected products by fixed amount',
        },
    ],
    args: {
        amount: {
            type: 'int',
            ui: {
                component: 'currency-form-input',
            },
        },
        productVariantIds: {
            type: 'ID',
            list: true,
            ui: { component: 'product-selector-form-input' },
            label: [{ languageCode: core_1.LanguageCode.en, value: 'Product variants' }],
        },
    },
    init(injector) {
        facetValueChecker = new core_1.FacetValueChecker(injector.get(core_1.TransactionalConnection));
    },
    async executeOnSubscription(ctx, currentSubscriptionPrice, orderLine, args) {
        if (lineContainsIds(args.productVariantIds, orderLine)) {
            const discount = -Math.min(args.amount, currentSubscriptionPrice); // make sure we don't discount more than the current price
            return discount;
        }
        return 0;
    },
});
exports.subscriptionPromotions = [
    exports.allByPercentage,
    exports.withFacetsByPercentage,
    exports.withFacetsByFixedAmount,
    exports.selectedProductsByFixedAmount,
    exports.selectedProductsByPercentage,
];
